#define _LARGEFILE64_SOURCE
#include <stdio.h>
#define __USE_GNU 1
#include <dlfcn.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>

#include "utils.h"


#define RANDOM_DEVICE "/dev/urandom"	/* use /dev/random for very secure: may lock the system */

long int get_seed(void)
{
	long int return_value;
	/* this will get the open of this lib, but since we're not
	 * doing any truncates here, that is safe
	 */
	int fd = open(RANDOM_DEVICE, O_RDONLY);

	if (fd != -1)
	{
		int rc = READ(fd, (char *)&return_value, sizeof(return_value));

		close(fd);

		if (rc == sizeof(return_value))
			return return_value;
	}

	syslog(LOG_ALERT, "libsd: alert! could not open " RANDOM_DEVICE " for random seed! (%m)");

	return_value = time(NULL) ^ getpid();

	return return_value;
}

long int limited_lrand48(long int max_val)
{
	/* not '%' as the low-order bits may be less random (like with 'rand()' on some systems) */
	return ( ((double)max_val) * ((double)lrand48()) ) / ((double)RAND_MAX);
}

ssize_t READ(int fd, char *whereto, size_t len)
{
	ssize_t cnt=0;

	while(len>0)
	{
		ssize_t rc;

		rc = read(fd, whereto, len);

		if (rc == -1)
		{
			if (errno != EINTR)
			{
				syslog(LOG_ERR, "libsd: error while read(%d, %p, %d): %m", fd, whereto, len);
				return -1;
			}
		}
		else if (rc == 0)
		{
			break;
		}
		else
		{
			whereto += rc;
			len -= rc;
			cnt += rc;
		}
	}

	return cnt;
}

ssize_t WRITE(int fd, char *whereto, size_t len)
{
	ssize_t cnt=0;

	while(len>0)
	{
		ssize_t rc;

		rc = write(fd, whereto, len);

		if (rc == -1)
		{
			if (errno != EINTR)
			{
				syslog(LOG_ERR, "libsd: error while read(%d, %p, %d): %m", fd, whereto, len);
				return -1;
			}
		}
		else if (rc == 0)
		{
			break;
		}
		else
		{
			whereto += rc;
			len -= rc;
			cnt += rc;
		}
	}

	return cnt;
}

void * retrieve_lib_function_pointer(const char *function_name)
{
	void *pnt;

	(void)dlerror();

	if ((pnt = dlsym(RTLD_NEXT, function_name)) == NULL)
	{
		char *err_str = dlerror();

		syslog(LOG_EMERG, "libsd: could not resolve '%s' in libraries: %s", function_name, err_str?err_str:"(no error?!)");
		_exit(1);
	}

	return pnt;
}

/* 0: file did NOT exist, 1: file DOES exist, -1: error */
int file_exists(const char *file_name)
{
	struct stat64 file_info;

	if (stat64(file_name, &file_info) == -1)
	{
		if (errno == ENOENT)
			return 0;

		syslog(LOG_ERR, "libsd: error while checking file %s: %m", file_name);

		return -1;
	}

	return 1;
}
